<?php
	/**
	 * @package     Joomla.Administrator
	 * @subpackage  com_formea
	 *
	 * @copyright   Copyright (C) 2010-2024 Feseur Sdn Bhd. All rights reserved.
	 * @license     GNU General Public License version 2 or later; see LICENSE.txt
	 * @version     1.1.6
	 */

	namespace Joomla\Component\Formea\Administrator\Controller;

	defined('_JEXEC') or die;

	use Joomla\CMS\Language\Text;
	use Joomla\CMS\MVC\Controller\AdminController;
	use Joomla\CMS\Router\Route;
	use Joomla\Component\Formea\Administrator\Model\FormeaModel;
	use function count;

	/**
	 * Formea list controller class.
	 *
	 * @since  1.0.0
	 */
	class FormeasController extends AdminController
	{
		/**
		 * The prefix to use with controller messages.
		 *
		 * @var    string
		 * @since  1.0.0
		 */
		protected $text_prefix = 'COM_FORMEA';


		/**
		 * Proxy for getModel.
		 *
		 * @param   string  $name    The name of the model.
		 * @param   string  $prefix  The prefix for the PHP class name.
		 * @param   array   $config  Array of configuration parameters.
		 *
		 * @return  FormeaModel
		 *
		 * @since   1.0.0
		 */
		public function getModel($name = 'Formea', $prefix = 'Administrator', $config = ['ignore_request' => true])
		{
			return parent::getModel($name, $prefix, $config);
		}

		public function clearSubmission()
		{
			$this->checkToken() or die;
			$cids             = $this->app->getInput()->getInt('cid', []);
			$model            = $this->getModel();
			$deleteSubmission = $model->deleteFormSubmissions($cids);
			if ($deleteSubmission)
			{
				$this->setMessage(Text::plural($this->text_prefix . '_N_FORMS_SUBMISSIONS_DELETED', count($cids)));
			}
			else
			{
				$this->setMessage($model->getError(), 'error');
			}
			$this->setRedirect(
				Route::_(
					'index.php?option=' . $this->option . '&view=' . $this->view_list
					. $this->getRedirectToListAppend(),
					false
				)
			);
		}
	}
