<?php
	/**
	 * @package     Joomla.Administrator
	 * @subpackage  com_formea
	 *
	 * @copyright   Copyright (C) 2010-2024 Feseur Sdn Bhd. All rights reserved.
	 * @license     GNU General Public License version 2 or later; see LICENSE.txt
	 * @version     1.1.6
	 */

	defined('_JEXEC') or die;

	use Joomla\CMS\Factory;
	use Joomla\CMS\Form\FormField;
	use Joomla\CMS\HTML\HTMLHelper;
	use Joomla\CMS\Language\Text;
	use Joomla\CMS\Layout\LayoutHelper;
	use Joomla\CMS\Router\Route;
	use Joomla\CMS\WebAsset\WebAssetManager;
	use Joomla\Component\Formea\Administrator\View\Formea\HtmlView;

	/** @var HtmlView $this */

	/** @var WebAssetManager $wa */
	$wa = $this->document->getWebAssetManager();
	$wa->useScript('keepalive')
		->useScript('form.validate')
		->useScript('field.modal-fields');

	//HTMLHelper::_('script', 'com_formea/admin-formeas-letter.js', array('version' => 'auto', 'relative' => true));

	$app                    = Factory::getApplication();
	$input                  = $app->input;
	$this->ignore_fieldsets = ['item_associations'];
	$this->useCoreUI        = true;

	// In case of modal
	$isModal   = $input->get('layout') == 'modal' ? true : false;
	$layout    = $isModal ? 'modal' : 'edit';
	$tmpl      = $isModal || $input->get('tmpl', '') === 'component' ? '&tmpl=component' : '';
	$totalLang = count($this->languageLists);
	$lTag      = str_replace('-', '_', $this->lang_tag);


	/* echo '<button type="button" class="btn btn-primary" data-bs-toggle="modal" data-bs-target="#modal-element">
	New Captcha
  </button>';*/

?>

<form
      action="<?php echo Route::_('index.php?option=com_formea&layout=' . $layout . $tmpl . '&id=' . (int) $this->item->id); ?>"
      method="post" name="adminForm" id="formea-form" class="form-validate">
   <div class="d-none hidden">
      <div>
         <input type="text" id="edit_element_id" placeholder="edit element id" class="form-control"
                oninput="FormeaElementIdChange()">
      </div>
      <div>
         <input type="text" id="edit_element_name" placeholder="edit element name" class="form-control">
      </div>
   </div>

   <div class="row">
      <div class="col-md-7">
		  <?php echo LayoutHelper::render('joomla.edit.title_alias', $this); ?>
      </div>
      <div class="col-md-5">
         <div class="row">
            <div class="col-md-6">
				<?php echo $this->getForm()->renderField('state'); ?>
            </div>
            <div class="col-md-6">
				<?php echo $this->getForm()->renderField('access_level'); ?>
            </div>
         </div>
      </div>
   </div>


   <div class="main-card mb-3 no-tab-padding">
	   <?php echo HTMLHelper::_('uitab.startTabSet', 'myTab', array('active' => 'layouts')); ?>

	   <?php echo HTMLHelper::_('uitab.addTab', 'myTab', 'layouts', Text::_('COM_FORMEA_FORM_LAYOUT')); ?>
      <div class="d-block pt-4">
		  <?php echo $this->form->getInput('elementlayout'); ?>
      </div>
	   <?php echo HTMLHelper::_('uitab.endTab');
		   echo HTMLHelper::_('uitab.addTab', 'myTab', 'details', Text::_('COM_FORMEA_FORM_INFO'));
		   echo '<div class="d-block p-3">';
		   echo $this->loadTemplate('info');
		   echo '</div>';
		   echo HTMLHelper::_('uitab.endTab');
		   echo HTMLHelper::_('uitab.addTab', 'myTab', 'theme', Text::_('COM_FORMEA_THEME'));
		   echo '<div class="d-block p-3">';
		   echo $this->loadTemplate('theme');
		   echo '</div>';
		   echo HTMLHelper::_('uitab.endTab');
		   echo HTMLHelper::_('uitab.addTab', 'myTab', 'form_submission', Text::_('COM_FORMEA_SUBMISSION_FORM'));
		   echo '<div class="d-block p-3">';
		   echo $this->loadTemplate('submission');
		   echo '</div>';
		   echo HTMLHelper::_('uitab.endTab');
		   echo HTMLHelper::_('uitab.addTab', 'myTab', 'submission_emails', Text::_('COM_FORMEA_EMAILS'));
		   echo '<div class="d-block p-3">';
		   echo $this->loadTemplate('emails');
		   echo '</div>';
		   echo HTMLHelper::_('uitab.endTab');
		   echo HTMLHelper::_('uitab.addTab', 'myTab', 'additional_style_script', Text::_('COM_FORMEA_CSS_JAVASCRIPT'));
		   echo '<div class="d-block p-3">';
		   echo $this->loadTemplate('cssjs');
		   echo '</div>';
		   echo HTMLHelper::_('uitab.endTab');
		   echo HTMLHelper::_('uitab.addTab', 'myTab', 'admin_columns', Text::_('COM_FORMEA_ADMIN_COLUMNS'));
		   echo '<div class="d-block p-3">';
		   echo $this->loadTemplate('admin_columns');
		   echo '</div>';
		   echo HTMLHelper::_('uitab.endTab');

		   if ($this->hasCoreExtensions)
		   {
			   echo HTMLHelper::_('uitab.addTab', 'myTab', 'cores', Text::_('COM_FORMEA_CORE_TAB'));
			   echo '<div class="d-block p-3">';
			   echo $this->loadTemplate('cores');
			   echo '</div>';
			   echo HTMLHelper::_('uitab.endTab');
		   }

	   ?>


	   <?php echo HTMLHelper::_('uitab.endTabSet'); ?>
   </div>
   <input type="hidden" name="task" value="">
	<?php
		echo $this->form->getInput('form_type');
		echo $this->form->getInput('renderer_id', 'params');
		echo $this->form->getInput('renderer_style_id', 'params');
		echo HTMLHelper::_('form.token');
	?>
</form>

<?php
	$title                      = Text::_('COM_FORMEA_ELEMENT');
	$link                       = 'index.php?option=com_formea&task=element.edit&type=JoomlaCaptcha&id=0&layout=modal&tmpl=component';
	$randomId                   = $title;
	$modalParams['title']       = $this->escape($title);
	$modalParams['footer']      = $this->loadTemplate('element_footer');
	$modalParams['url']         = $link;
	$modalParams['backdrop']    = 'static';
	$modalParams['keyboard']    = false;
	$modalParams['closeButton'] = false;
	$modalParams['height']      = '100%';
	$modalParams['width']       = '100%';
	$modalParams['bodyHeight']  = 70;
	$modalParams['modalWidth']  = 80;
	echo HTMLHelper::_('bootstrap.renderModal', 'modal-element', $modalParams);
?>


