<?php
	/**
	 * @package     Joomla.Administrator
	 * @subpackage  com_formea
	 *
	 * @copyright   Copyright (C) 2010-2024 Feseur Sdn Bhd. All rights reserved.
	 * @license     GNU General Public License version 2 or later; see LICENSE.txt
	 * @version     1.1.6
	 */

  defined('_JEXEC') or die;

  use Feseur\Library\FsrDate;
  use Joomla\CMS\Component\ComponentHelper;
  use Joomla\CMS\Factory;
  use Joomla\CMS\HTML\HTMLHelper;
  use Joomla\CMS\Language\Text;
  use Joomla\CMS\Router\Route;
  use Joomla\CMS\User\User;
  use Joomla\CMS\User\UserFactoryInterface;
  use Joomla\CMS\WebAsset\WebAssetManager;
  use Joomla\Component\Formea\Administrator\View\Validation\HtmlView;

  /** @var HtmlView $this */

  /** @var WebAssetManager $wa */
  $wa = $this->document->getWebAssetManager();
  $wa->useScript('keepalive');

  $app   = Factory::getApplication();
  $input = $app->input;

  // In case of modal
  $isModal       = $input->get('layout') == 'modal' ? true : false;
  $layout        = $isModal ? 'modal' : 'edit';
  $tmpl          = $isModal || $input->get('tmpl', '') === 'component' ? '&tmpl=component' : '';
  $totalDetails  = count($this->item->details);
  $dateSubmitted = new FsrDate($this->item->created_date);
  $formeaConfig  = ComponentHelper::getParams('com_formea');

?>
<form
  action="<?php echo Route::_('index.php?option=com_formea&view=submission&layout=' . $layout . $tmpl . '&id=' . (int) $this->item->id); ?>"
  method="post" name="adminForm" id="formea-form" class="form-validate">
  <div class="row gx-2">
    <div class="col-md-8">
      <table class="table rounded">
        <?php for ($i = 0; $i < $totalDetails; $i++) { ?>
          <tr>
            <th><?php echo $this->item->details[$i]->field_caption; ?></th>
            <td colspan="3"><?php echo $this->item->details[$i]->field_value; ?></td>
          </tr>
        <?php } ?>
      </table>
    </div>
    <div class="col-md-4">
      <div class="list-group">
        <div class="list-group-item d-flex justify-content-between gap-2 align-items-center">
          <div class="fs-sm text-muted"><?php echo Text::_('COM_FORMEA_FORM_NAME'); ?></div>
          <div><?php echo $this->item->form_title; ?></div>
        </div>
        <div class="list-group-item d-flex justify-content-between gap-2 align-items-center">
          <div class="fs-sm text-muted"><?php echo Text::_('COM_FORMEA_SUBMITTED_BY'); ?></div>
          <div><?php
              if ($this->item->user_id < 1)
              {
                echo Text::_('COM_FORMEA_GUEST');
              }
              else
              {
                /** @var User $submitter */
                $submitter = Factory::getContainer()->get(UserFactoryInterface::class)->loadUserById($this->item->user_id);
                echo $submitter->get('name');
              }
            ?>
          </div>
        </div>
        <div class="list-group-item d-flex justify-content-between gap-2 align-items-center">
          <div class="fs-sm text-muted"><?php echo Text::_('COM_FORMEA_DATE_SUBMITTED'); ?></div>
          <div><?php echo $dateSubmitted->format($formeaConfig->get('date_format', 'F j, Y H:i:s')); ?></div>
        </div>
        <div class="list-group-item d-flex justify-content-between gap-2 align-items-center">
          <div class="fs-sm text-muted"><?php echo Text::_('COM_FORMEA_IP_ADDRESS'); ?></div>
          <div><?php echo $this->item->ip_address; ?></div>
        </div>
        <div class="list-group-item d-flex justify-content-between gap-2 align-items-center">
          <div class="fs-sm text-muted"><?php echo Text::_('COM_FORMEA_LANGUAGE'); ?></div>
          <div><?php echo $this->item->language; ?></div>
        </div>
      </div>
    </div>
  </div>
  <input type="hidden" name="task" value="">
  <?php echo HTMLHelper::_('form.token'); ?>
</form>
