<?php

	/**
	 * This is a forked library modified for use in Formea extension,
	 * Credit goes to original author(s) Eduardo Gulias Davis at https://github.com/egulias/EmailValidator.
	 * See LICENSE
	 *
	 * @version     1.0.0
	 * @package     Formea Email Validation
	 * @subpackage  Lexer
	 * @author      Feseur Sdn Bhd
	 * @copyright   Copyright (C) 2010-2024 Feseur Sdn Bhd. All rights reserved.
	 * @license     GNU General Public License version 2 or later; see LICENSE.txt
	 */

	namespace Egulias\EmailValidator;

	use Egulias\EmailValidator\Result\Result;
	use Egulias\EmailValidator\Result\ValidEmail;
	use Egulias\EmailValidator\Result\InvalidEmail;
	use Egulias\EmailValidator\Result\Reason\ExpectingATEXT;

	abstract class Parser
	{
		/**
		 * @var Warning\Warning[]
		 */
		protected $warnings = [];

		/**
		 * @var EmailLexer
		 */
		protected $lexer;

		/**
		 * id-left "@" id-right
		 */
		abstract protected function parseRightFromAt(): Result;

		abstract protected function parseLeftFromAt(): Result;

		abstract protected function preLeftParsing(): Result;


		public function __construct(EmailLexer $lexer)
		{
			$this->lexer = $lexer;
		}

		public function parse(string $str): Result
		{
			$this->lexer->setInput($str);

			if ($this->lexer->hasInvalidTokens())
			{
				return new InvalidEmail(new ExpectingATEXT("Invalid tokens found"), $this->lexer->token["value"]);
			}

			$preParsingResult = $this->preLeftParsing();
			if ($preParsingResult->isInvalid())
			{
				return $preParsingResult;
			}

			$localPartResult = $this->parseLeftFromAt();

			if ($localPartResult->isInvalid())
			{
				return $localPartResult;
			}

			$domainPartResult = $this->parseRightFromAt();

			if ($domainPartResult->isInvalid())
			{
				return $domainPartResult;
			}

			return new ValidEmail();
		}

		/**
		 * @return Warning\Warning[]
		 */
		public function getWarnings(): array
		{
			return $this->warnings;
		}

		protected function hasAtToken(): bool
		{
			$this->lexer->moveNext();
			$this->lexer->moveNext();

			return ((array) $this->lexer->token)['type'] !== EmailLexer::S_AT;
		}
	}
