<?php

	/**
	 * @package     Joomla.Site
	 * @subpackage  com_formea
	 *
	 * @copyright   Copyright (C) 2010-2024 Feseur Sdn Bhd. All rights reserved.
	 * @license     GNU General Public License version 2 or later; see LICENSE.txt
	 * @version     1.1.6
	 */

	namespace Joomla\Component\Formea\Site\Service;

	defined('_JEXEC') or die;

	use Joomla\CMS\Application\SiteApplication;
	use Joomla\CMS\Categories\CategoryFactoryInterface;
	use Joomla\CMS\Component\ComponentHelper;
	use Joomla\CMS\Component\Router\RouterView;
	use Joomla\CMS\Component\Router\RouterViewConfiguration;
	use Joomla\CMS\Component\Router\Rules\MenuRules;
	use Joomla\Component\Formea\Site\Service\FormeaNoMenuRules;
	use Joomla\CMS\Component\Router\Rules\StandardRules;
	use Joomla\CMS\Menu\AbstractMenu;
	use Joomla\Database\DatabaseInterface;
	use Joomla\Database\ParameterType;

	/**
	 * Routing class of com_specialties
	 *
	 * @since  3.3
	 */
	class Router extends RouterView
	{
		protected $noIDs = false;

		/**
		 * The category factory
		 *
		 * @var CategoryFactoryInterface
		 *
		 * @since  4.0.0
		 */
		private $categoryFactory;

		/**
		 * The db
		 *
		 * @var DatabaseInterface
		 *
		 * @since  4.0.0
		 */
		private $db;

		/**
		 * Mywalks Component router constructor
		 *
		 * @param   SiteApplication           $app              The application object
		 * @param   AbstractMenu              $menu             The menu object to work with
		 * @param   CategoryFactoryInterface  $categoryFactory  The category object
		 * @param   DatabaseInterface         $db               The database object
		 *
		 * @since 1.0.0
		 */
		public function __construct(SiteApplication          $app, AbstractMenu $menu,
		                            CategoryFactoryInterface $categoryFactory, DatabaseInterface $db)
		{
			$this->db = $db;
			$formea   = new RouterViewConfiguration('formea');
			$formea->setKey('id');
			$this->registerView($formea);

			parent::__construct($app, $menu);

			$this->attachRule(new MenuRules($this));
			$this->attachRule(new StandardRules($this));
			$this->attachRule(new FormeaNoMenuRules($this));
		}

		public function getFormeaSegment($id, $query)
		{
			if (!strpos($id, ':'))
			{
				$id      = (int) $id;
				$dbquery = $this->db->getQuery(true);
				$dbquery->select($this->db->quoteName('alias'))
					->from($this->db->quoteName('#__formea_forms'))
					->where($this->db->quoteName('id') . ' = :id')
					->bind(':id', $id, ParameterType::INTEGER);
				$this->db->setQuery($dbquery);

				$id .= ':' . $this->db->loadResult();
			}

			if ($this->noIDs)
			{
				list($void, $segment) = explode(':', $id, 2);

				return [$void => $segment];
			}

			return [(int) $id => $id];
		}
	}
